# encoding: utf-8

# ------------------------------------------------------------------------------
# Copyright (c) 2016 SUSE Linux GmbH, Nuernberg, Germany.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact SUSE Linux GmbH.
#
# ------------------------------------------------------------------------------
#
# Summary: SUSE Connect Program: tests for the Product class
# Authors: Ilya Manyugin <ilya.manyugin@suse.com>

require_relative 'spec_helper'

require 'suse_connect_program/product_controller'
require 'suse_connect_program/product_manager'

describe SuseConnectProgram::ProductController do
  before do
    allow(File).to receive(:read).and_call_original
    allow(File).to receive(:read).with('/usr/share/YaST2/data/suse_connect_program/template_gui.erb')
      .and_return(template('template_gui.erb'))
    allow(File).to receive(:read).with('/usr/share/YaST2/data/suse_connect_program/template_ncurses.erb')
      .and_return(template('template_ncurses.erb'))
  end

  describe '#prepare' do
    before do
      expect(SuseConnectProgram::ProductManager.instance).to receive(:lock) { raise RuntimeError }
    end
    it 'throws an error when no package lock can be acquired' do
      pc = SuseConnectProgram::ProductController.new(file_path('products_single_valid.json'), true)
      expect { pc.prepare }.to raise_error(SuseConnectProgram::ProductControllerGUIError)
    end
  end

  context 'when given a file with a single valid product' do
    describe '.initialize' do
      subject { SuseConnectProgram::ProductController.new(file_path('products_single_valid.json'), true) }

      before do
        expect(SuseConnectProgram::ProductManager.instance).to receive(:lock)
      end

      it 'reads all entries from the file' do
        subject.prepare
        expect(subject.filter_product_list('').size).to eq 1
      end

      it 'returns an empty list if the search string is not a correct RegExp' do
        subject.prepare
        expect(subject.filter_product_list('((.*)').size).to eq 0
      end

      it 'sorts out the products not matching the search terms' do
        subject.prepare
        expect(subject.filter_product_list('PRODUCT 1').size).to eq 1
        expect(subject.filter_product_list('PRODUCT 999').size).to eq 0
      end
    end
  end

  context 'when given a correct product list' do
    subject { SuseConnectProgram::ProductController.new(file_path('products_single_valid.json'), true) }

    before do
      expect(SuseConnectProgram::ProductManager.instance).to receive(:lock)
      subject.prepare
      expect(SuseConnectProgram::ProductManager.instance).to receive(:repository_added?)
        .with(any_args).and_return(true)
      expect(SuseConnectProgram::ProductManager.instance).to receive(:resolvable_installed?)
        .with(any_args).and_return(true)
    end

    it 'gives the correct information about the product' do
      product = subject.get_product_details(0)
      expect(product.repo_added).to be true
      expect(product.installed).to be true
    end
  end

  context 'when requesting a non-existent product by its id' do
    subject { SuseConnectProgram::ProductController.new(file_path('products_single_valid.json'), true) }

    before do
      expect(SuseConnectProgram::ProductManager.instance).to receive(:lock)
      subject.prepare
    end
    
    it 'returns nil' do
      expect(SuseConnectProgram::ProductManager.instance).to receive(:lock)
      subject.prepare
      expect(subject.get_product_details(1000)).to eq nil
    end
  end

  context 'when there is a network error' do
    subject { SuseConnectProgram::ProductController.new(file_path('products_single_valid.json'), true) }
    before do
      expect(SuseConnectProgram::ProductManager.instance).to receive(:lock)
      subject.prepare
      expect(Net::HTTP).to receive(:get_response).and_raise(SocketError)
    end

    it 'throws a GUI exception' do
      expect { subject.filter_product_list('').length.to eq 1 }
      expect { subject.add_repository(0) }
        .to raise_exception(SuseConnectProgram::ProductControllerGUIError)
    end
  end
end
