# encoding: utf-8

# ------------------------------------------------------------------------------
# Copyright (c) 2016 SUSE Linux GmbH, Nuernberg, Germany.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact SUSE Linux GmbH.
#
# ------------------------------------------------------------------------------
#
# Summary: SUSE Connect Program: tests for the Product class
# Authors: Ilya Manyugin <ilya.manyugin@suse.com>

require_relative 'spec_helper'
require 'suse_connect_program/product_manager'
require 'net/http'

describe SuseConnectProgram::ProductManager do
  before do
    allow(File).to receive(:read).and_call_original
    allow(File).to receive(:read).with('/usr/share/YaST2/data/suse_connect_program/template_gui.erb')
      .and_return(template('template_gui.erb'))
    allow(File).to receive(:read).with('/usr/share/YaST2/data/suse_connect_program/template_ncurses.erb')
      .and_return(template('template_ncurses.erb'))
  end
  subject { SuseConnectProgram::ProductManager.instance }

  let (:repositories) do
    [
      {
       'alias'=>'repo-update',
       'autorefresh'=>true,
       'base_urls'=>['http://download.opensuse.org/update/leap/42.1/oss/'],
       'enabled'=>true,
       'is_update_repo'=>false,
       'keeppackages'=>false,
       'mirror_list'=>'',
       'name'=>'openSUSE-Leap-42.1-Update',
       'priority'=>99,
       'product_dir'=>'/',
       'raw_url'=>'http://download.opensuse.org/update/leap/42.1/oss/',
       'service'=>'',
       'type'=>'YUM',
       'url'=>'http://download.opensuse.org/update/leap/42.1/oss/'
      }
    ]
  end

  let (:resolvable_property) do
    [{'arch'=>'x86_64',
      'description'=>'The GNU C Library provides the most important standard libraries...',
      'download_size'=>0,
      'inst_size'=>6653961,
      'locked'=>false,
      'medium_nr'=>0,
      'name'=>'glibc',
      'on_system_by_user'=>true,
      'source'=>-1,
      'status'=>:installed,
      'summary'=>'Standard Shared Libraries (from the GNU C Library)',
      'transact_by'=>:solver,
      'vendor'=>'openSUSE',
      'version'=>'2.19-19.1',
      'version_epoch'=>nil,
      'version_release'=>'19.1',
      'version_version'=>'2.19'}
    ]
  end

  describe "#lock" do
    before do
      allow(Yast::PackageLock).to receive(:Connect).and_return('connected' => connected)
    end

    context 'when the libzypp lock can be obtained' do
      let(:connected) { true }
      it 'initializes package management' do
        expect(Yast::PackageCallbacks).to receive(:InitPackageCallbacks)
        expect(Yast::Pkg).to receive(:TargetInitialize).and_return(true)
        expect(Yast::Pkg).to receive(:TargetLoad).and_return(true)
        expect(Yast::Pkg).to receive(:SourceStartManager).with(true).and_return(true)
        expect(Yast::KeyManager).to receive(:Read)
        expect(subject).to receive(:refresh_sources)
        subject.lock
      end
    end

    context 'when the lock cannot be obtained' do
      let(:connected) { false }
      it 'raises an exception' do
        expect { subject.lock }.to raise_error(RuntimeError)
      end
    end

  end

  describe '#add_repository' do
    let(:connected) { true }
    let(:repo_url) { repositories[0]['base_urls'][0] }
    let(:repo_props) do
      {
        'enabled' => true,
        'autorefresh' => true,
        'name' => 'Project Argus',
        'prod_dir' => '/',
        'alias' => 'project_argus',
        'base_urls' => [repo_url],
        'type' => 'YUM'
      }
    end

    context 'when given a YUM repository' do
      before do
        expect(Yast::Pkg).to receive(:RepositoryProbe)
          .with(repo_url, '').and_return('YUM')
        expect(Yast::Pkg).to receive(:RepositoryAdd).with(repo_props).and_return(0)
        expect(Yast::Pkg).to receive(:SourceGeneralData).with(0).and_return(repositories[0])
        expect(Yast::Pkg).to receive(:SourceRefreshNow).with(0).and_return(true)
        expect(Yast::KeyManager).to receive(:Write)
        expect(Yast::Pkg).to receive(:SourceSaveAll)
        expect(subject).to receive(:refresh_sources)
      end

      it 'throws an error' do
        # since we cannot actually add the repository in our tests
        expect { subject.add_repository(repo_url, 'project_argus', 'Project Argus') }
          .to raise_exception(SuseConnectProgram::ProductManagerException)
      end
    end

    context 'when given a non-YUM repository' do
      before do
        expect(Yast::Pkg).to receive(:RepositoryProbe).with(repo_url, '').and_return(nil)
      end

      it 'throws an exception' do
        expect { subject.add_repository(repo_url, 'project_argus', 'Project Argus') }
          .to raise_exception(SuseConnectProgram::ProductManagerException)
      end
    end
  end

  describe '#remove_repository' do
    let(:connected) { true }
    let(:repo_url) { repositories[0]['base_urls'][0] }
    let(:source_id) { 100 }

    context "given that the repository wasn't added" do
      before do
        expect(subject).to receive(:source_for_url).with(repo_url).and_return(nil)
      end

      it 'returns true' do
        expect(subject.remove_repository(repo_url)).to be true
      end
    end

    context 'given that the repository was added before' do
      before do
        expect(subject).to receive(:source_for_url).with(repo_url).and_return(source_id)
        expect(Yast::Pkg).to receive(:SourceDelete).with(source_id).and_return(true)
      end

      it 'returns true' do
        expect(subject.remove_repository(repo_url)).to be true
      end
    end
  end

  describe '#install_resolvable' do
    it 'raises an error when no suitable resolvable is found' do
      expect(subject).to receive(:find_resolvables).with('test_package').and_return([:package, []])
      expect { subject.install_resolvable('test_package', repositories[0]['base_urls'][0]) }
        .to raise_error(RuntimeError)
    end
  end
end
